@@ -11,9 +11,13 @@
 
 import ctypes
 import io
+import operator
 import os
 import sys
 import weakref
+from functools import reduce
+from pathlib import Path
+from tempfile import NamedTemporaryFile
 
 from . import _check_status, _keepref, cairo, constants, ffi
 from .fonts import FontOptions, _encode_string
@@ -76,6 +80,10 @@ def from_buffer(obj):
         # Looks like a array.array object.
         address, length = obj.buffer_info()
         return address, length * obj.itemsize
+    elif hasattr(obj, '__array_interface__'):
+        # Looks like a numpy.ndarray object
+        length = reduce(operator.mul, obj.shape)
+        return ctypes.addressof(ctypes.c_char.from_buffer(obj)), length
     else:
         # Other buffers.
         # XXX Unfortunately ctypes.c_char.from_buffer
@@ -148,7 +156,11 @@ class Surface(object):
         self._pointer = ffi.gc(
             pointer, _keepref(cairo, cairo.cairo_surface_destroy))
         self._check_status()
-        if target_keep_alive not in (None, ffi.NULL):
+        if hasattr(target_keep_alive, '__array_interface__'):
+            is_empty = target_keep_alive.size == 0
+        else:
+            is_empty = target_keep_alive in (None, ffi.NULL)
+        if not is_empty:
             keep_alive = KeepAlive(target_keep_alive)
             _check_status(cairo.cairo_surface_set_user_data(
                 self._pointer, SURFACE_TARGET_KEY, *keep_alive.closure))
@@ -643,9 +655,25 @@ class Surface(object):
         if return_bytes:
             target = io.BytesIO()
         if hasattr(target, 'write'):
-            write_func = _make_write_func(target)
-            _check_status(cairo.cairo_surface_write_to_png_stream(
-                self._pointer, write_func, ffi.NULL))
+            try:
+                write_func = _make_write_func(target)
+                _check_status(cairo.cairo_surface_write_to_png_stream(
+                    self._pointer, write_func, ffi.NULL))
+            except SystemError:  # noqa
+                # Callback creation has failed
+                if hasattr(target, 'name'):
+                    # File-like object has a name, write here
+                    _check_status(cairo.cairo_surface_write_to_png(
+                        self._pointer, _encode_filename(target.name)))
+                else:
+                    # Use a fallback temporary file
+                    with NamedTemporaryFile('wb', delete=False) as fd:
+                        filename = fd.name
+                        _check_status(cairo.cairo_surface_write_to_png(
+                            self._pointer, _encode_filename(filename)))
+                    png_file = Path(filename)
+                    target.write(png_file.read_bytes())
+                    png_file.unlink()
         else:
             _check_status(cairo.cairo_surface_write_to_png(
                 self._pointer, _encode_filename(target)))
