@@ -255,7 +255,9 @@ def test_metadata():
     assert b'/Creator (creator)' in pdf_bytes
     assert b'/Author (author)' in pdf_bytes
     assert b'/Keywords (keywords)' in pdf_bytes
-    assert b"/CreationDate (20130721234600+01'00)" in pdf_bytes
+    # cairo 1.17.4 adds an apostrophe at the end of dates:
+    # https://gitlab.freedesktop.org/cairo/cairo/-/issues/392
+    assert b"/CreationDate (20130721234600+01'00" in pdf_bytes
     assert b'/ModDate (20130721234600Z)' in pdf_bytes
 
 
